-- blk_id_decode32.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- 	*** revision history ***

-- 10 Jan 2002:	take the general blk_id_decode module used on MAX, HIGH, and MED 
--				and expand it to handle 32-bit input.
-- 08 Aug 2002: reduce extended Pt bin bus to 3 bits.

-- 	what duz this do?  A "one hot" blk_id word is supplied to this module.  this module decodes 
--  it into a PHI_BASE (0 or 21) and some other stuff:

-- VALID		valid track bit.  Set if the blk_id word has at least 1 set bit in it.
--
-- CURVE		track curvature bit.  set if the track curves in a CCW direction
--				when the detector is viewed "LOOKING NORTH at the SOUTH face of the
--				detector".
--
-- PT_SUB		pt sub-bin. See below.

-- 1x22 blocks are arranged in the 44x16 Matrix as follows:

--  44444333333333322222222221111111111000000000
--  43210987654321098765432109876543210987654321    Curvature Bit   Pt-Sub Value
-- +--------------------------------------------+   -------------   ------------
-- |[-------  1  --------][-------- 0 ---------]|   CCW 1              0
-- |[-------  3  --------][-------- 2 ---------]|   CW 0               0
-- |[-------  5  --------][-------- 4 ---------]|   CCW 1              1
-- |[-------  7  --------][-------- 6 ---------]|   CW 0               1
-- |[-------  9  --------][-------- 8 ---------]|   CCW 1              2
-- |[------- 11  --------][------- 10 ---------]|   CW 0               2
-- |[------- 13  --------][------- 12 ---------]|   CCW 1              3
-- |[------- 15  --------][------- 14 ---------]|   CW 0               3
-- |[------- 17  --------][------- 16 ---------]|   CCW 1              4
-- |[------- 19  --------][------- 18 ---------]|   CW 0               4
-- |[------- 21  --------][------- 20 ---------]|   CCW 1              5
-- |[------- 23  --------][------- 22 ---------]|   CW 0               5
-- |[------- 25  --------][------- 24 ---------]|   CCW 1              6
-- |[------- 27  --------][------- 26 ---------]|   CW 0               6
-- |[------- 29  --------][------- 28 ---------]|   CCW 1              7
-- |[------- 31  --------][------- 30 ---------]|   CW 0               7
-- +--------------------------------------------+


library ieee;
use ieee.std_logic_1164.all;

entity blk_id_decode32 is
 	port (
		blk_id:		in 	std_logic_vector(31 downto 0);
		clock:		in 	std_logic;

		valid:		out	std_logic;
		muon_valid:	out	std_logic;
		curve:		out	std_logic;
		pt_sub:		out	std_logic_vector(2 downto 0);
		phi_base: 	out std_logic_vector(4 downto 0)	
	);
end blk_id_decode32;


architecture block_id_arch32 of blk_id_decode32 is

	-- define registers

	signal block_id_reg: 	std_logic_vector(31 downto 0);
	signal curve_reg: 		std_logic;
	signal valid_reg: 		std_logic;
	signal pt_sub_reg: 		std_logic_vector(2 downto 0);
	signal phi_base_reg: 	std_logic_vector(4 downto 0);

	-- signals

	signal temp_sub:		std_logic_vector(2 downto 0);
	signal temp_phi:		std_logic_vector(4 downto 0);
	signal temp_val:		std_logic;
	signal temp_curve:		std_logic;
	--signal temp_offset:		std_logic_vector(2 downto 0);


 begin


	-- combinatorial stuff

	curve 					<= curve_reg;
	valid					<= valid_reg;
	pt_sub(2 downto 0)		<= pt_sub_reg(2 downto 0);
	phi_base(4 downto 0) 	<= phi_base_reg(4 downto 0);

	temp_val				<= 	block_id_reg(31) or block_id_reg(30) or
								block_id_reg(29) or block_id_reg(28) or
								block_id_reg(27) or block_id_reg(26) or
								block_id_reg(25) or block_id_reg(24) or
								block_id_reg(23) or block_id_reg(22) or
								block_id_reg(21) or block_id_reg(20) or
								block_id_reg(19) or block_id_reg(18) or
								block_id_reg(17) or block_id_reg(16) or
								block_id_reg(15) or block_id_reg(14) or
								block_id_reg(13) or block_id_reg(12) or
								block_id_reg(11) or block_id_reg(10) or
								block_id_reg( 9) or block_id_reg( 8) or
								block_id_reg( 7) or block_id_reg( 6) or
								block_id_reg( 5) or block_id_reg( 4) or
								block_id_reg( 3) or block_id_reg( 2) or
								block_id_reg( 1) or block_id_reg( 0);								

	temp_curve				<= 	block_id_reg(29) or block_id_reg(28) or
								block_id_reg(25) or block_id_reg(24) or
								block_id_reg(21) or block_id_reg(20) or
								block_id_reg(17) or block_id_reg(16) or
								block_id_reg(13) or block_id_reg(12) or
								block_id_reg( 9) or block_id_reg( 8) or
								block_id_reg( 5) or block_id_reg( 4) or
								block_id_reg( 1) or block_id_reg( 0);

	temp_sub(2 downto 0) <=  	"000" when (block_id_reg( 0)='1' or block_id_reg( 1)='1') else 
								"000" when (block_id_reg( 2)='1' or block_id_reg( 3)='1') else 
								"001" when (block_id_reg( 4)='1' or block_id_reg( 5)='1') else 
								"001" when (block_id_reg( 6)='1' or block_id_reg( 7)='1') else 
								"010" when (block_id_reg( 8)='1' or block_id_reg( 9)='1') else 
								"010" when (block_id_reg(10)='1' or block_id_reg(11)='1') else
								"011" when (block_id_reg(12)='1' or block_id_reg(13)='1') else
								"011" when (block_id_reg(14)='1' or block_id_reg(15)='1') else
								"100" when (block_id_reg(16)='1' or block_id_reg(17)='1') else
								"100" when (block_id_reg(18)='1' or block_id_reg(19)='1') else
								"101" when (block_id_reg(20)='1' or block_id_reg(21)='1') else
								"101" when (block_id_reg(22)='1' or block_id_reg(23)='1') else
								"110" when (block_id_reg(24)='1' or block_id_reg(25)='1') else
								"110" when (block_id_reg(26)='1' or block_id_reg(27)='1') else
								"111";

	temp_phi(4 downto 0) <=		"10110" when (block_id_reg( 1)='1' or block_id_reg( 3)='1') else
								"10110" when (block_id_reg( 5)='1' or block_id_reg( 7)='1') else
								"10110" when (block_id_reg( 9)='1' or block_id_reg(11)='1') else
								"10110" when (block_id_reg(13)='1' or block_id_reg(15)='1') else
								"10110" when (block_id_reg(17)='1' or block_id_reg(19)='1') else
								"10110" when (block_id_reg(21)='1' or block_id_reg(23)='1') else
								"10110" when (block_id_reg(25)='1' or block_id_reg(27)='1') else
								"10110" when (block_id_reg(29)='1' or block_id_reg(31)='1') else
								"00000";

	-- sequential stuff

		process(clock)
		 begin
			if rising_edge(clock) then
				block_id_reg(31 downto 0) 	<= blk_id(31 downto 0);
				valid_reg					<= temp_val;		
				muon_valid					<= temp_val;		
				curve_reg					<= temp_curve;
				pt_sub_reg(2 downto 0) 		<= temp_sub(2 downto 0);
				phi_base_reg(4 downto 0) 	<= temp_phi(4 downto 0);
			end if;
		end process;
	
end block_id_arch32;
