library IEEE;
-- ddr_demux works in pair with ddr_mux to move data between two FPGAs
-- using DDR I/O register at twice the clock rate to reduce interconnects.
-- link_a into ddr_mux comes out as q_a
-- link_b into ddr_mux comes out as q_b
-- the net delay inserted is one clock cycle
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ddr_demux is
    Port ( clk : in std_logic;
           clkn : in std_logic;
	        load : in std_logic;
           ddr_link : in std_logic_vector(27 downto 0);
           q_a : out std_logic_vector(195 downto 0);
           q_b : out std_logic_vector(195 downto 0);
			  link_a : out std_logic_vector(27 downto 0);
			  link_b : out std_logic_vector(27 downto 0)
			 );
end ddr_demux;

architecture my_arch of ddr_demux is
signal din_a: std_logic_vector(27 downto 0);
signal din_b: std_logic_vector(27 downto 0);
signal sr_a: std_logic_vector(167 downto 0);
signal sr_b: std_logic_vector(167 downto 0);
signal load_a: std_logic;
signal load_b: std_logic;
attribute IOB : string;
attribute IOB of din_a, din_b: signal is "TRUE";
begin
process(clk)
begin
  if(clk'event and clk = '1')then
    din_b <= ddr_link;
  end if;
end process;
process(clkn)
begin
  if(clkn'event and clkn = '1')then
    link_a <= sr_a(27 downto 0);
    link_b <= sr_b(27 downto 0);
    din_a <= ddr_link;
	 sr_a(167 downto 140) <= din_a;
	 sr_a(139 downto 112) <= sr_a(167 downto 140);
	 sr_a(111 downto 84) <= sr_a(139 downto 112);
	 sr_a(83 downto 56) <= sr_a(111 downto 84);
	 sr_a(55 downto 28) <= sr_a(83 downto 56);
	 sr_a(27 downto 0) <= sr_a(55 downto 28);
	 sr_b(167 downto 140) <= din_b;
	 sr_b(139 downto 112) <= sr_b(167 downto 140);
	 sr_b(111 downto 84) <= sr_b(139 downto 112);
	 sr_b(83 downto 56) <= sr_b(111 downto 84);
	 sr_b(55 downto 28) <= sr_b(83 downto 56);
	 sr_b(27 downto 0) <= sr_b(55 downto 28);
	 load_a <= load;
	 load_b <= load;
	 if(load_a = '1')then
	   q_a <= din_a & sr_a;
	 end if;
	 if(load_b = '1')then
	   q_b <= din_b & sr_b;
	 end if;
  end if;
end process;
end my_arch;
