-- low_dpr_load.vhd
-- part of the "expandable" backend pipeline LOWEST specific
-- takes two tracks and puts them into two dual port RAMs.

-- 1. if the trk is not valid (MSb=0) write zeros into the address pointed to by the pointer.  
-- 2. if same=1 write ('0'&trk_a) into ram_a and increment ptr_a.
-- 3. if same=0 write ('1'&trk_a) into ram_a and incr ptr_A.  on the same clk tick, write ('0'&trk_b) into 
--	ram_b and incr ptr_b.

-- modified 8 August 2002.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity low_dpr_load is
 	port (
		trk_a:		in 	std_logic_vector(10 downto 0);	-- VCEeePppppp
		trk_b:		in	std_logic_vector(10 downto 0);
		same:		in 	std_logic;
		clock:		in 	std_logic;
		start:		in	std_logic;						-- active high, set for 1st word of the frame

		data_a:		out	std_logic_vector(11 downto 0);	-- data going into rams SVCEeePppppp
		data_b:		out	std_logic_vector(11 downto 0);
		ptr_a:		out	std_logic_vector(3 downto 0);	-- address pointers
		ptr_b:		out	std_logic_vector(3 downto 0)
		
	);
end low_dpr_load;


architecture low_dpr_load_arch of low_dpr_load is

	-- define registers...

		signal ptr_a_reg: 		std_logic_vector( 3 downto 0);
		signal ptr_b_reg: 		std_logic_vector( 3 downto 0);
		signal trk_a_reg:		std_logic_vector(10 downto 0);
		signal trk_b_reg:		std_logic_vector(10 downto 0);
		signal data_a_reg:		std_logic_vector(11 downto 0);
		signal data_b_reg:		std_logic_vector(11 downto 0);
		signal same_reg:		std_logic;
		signal start_reg:		std_logic;
	
	-- define signals...

		signal temp_a:			std_logic_vector(11 downto 0);
		signal temp_b:			std_logic_vector(11 downto 0);

	-- declare components...

begin

	-- combinatorial stuff...

		data_a(11 downto 0) <= data_a_reg(11 downto 0);		
		data_b(11 downto 0) <= data_b_reg(11 downto 0);
		ptr_a(3 downto 0)	<= ptr_a_reg(3 downto 0);
		ptr_b(3 downto 0)	<= ptr_b_reg(3 downto 0);

		temp_a(11 downto 0)	<= 	('1' & trk_a_reg(10 downto 0)) when (same_reg='0' and trk_a_reg(10)='1') else
								('0' & trk_a_reg(10 downto 0)) when (same_reg='1' and trk_a_reg(10)='1') else
								"000000000000";

		temp_b(11 downto 0)	<= 	('0' & trk_b_reg(10 downto 0)) when (trk_b_reg(10)='1') else
								"000000000000";

	-- sequential stuff...

		process(clock)
		 begin

			if rising_edge(clock) then

				data_a_reg(11 downto 0) <= temp_a(11 downto 0);
				data_b_reg(11 downto 0) <= temp_b(11 downto 0);

				trk_a_reg(10 downto 0) 	<= trk_a(10 downto 0);
				trk_b_reg(10 downto 0) 	<= trk_b(10 downto 0);
				same_reg				<= same;
				start_reg				<= start;

				if (start_reg='1') then
					ptr_a_reg(3 downto 0) <= "0000";
					ptr_b_reg(3 downto 0) <= "0000";
				else
					ptr_a_reg(3 downto 0) <= ptr_a_reg(3 downto 0) + 1;
					ptr_b_reg(3 downto 0) <= ptr_b_reg(3 downto 0) + 1;
				end if;

			end if;
		end process;


	-- instantiate components...

end low_dpr_load_arch;
