-- low pt track equations
-- doublets in, matrix out.  fully combinatorial, no registers.
-- optimize for small area when compiling

-- jamieson olsen <jamieson@fnal.gov> 
-- Automated version for as-built geometry 
--   Graham W. Wilson <graham@fnal.gov>  March 12th 2002
--            eqn IDs added     March 4th 2004
-- Input file  : low_tsector05.sorted                    
-- Output file : low_tsector05.vhdl                      

-- low design, full sector

library ieee;
  use ieee.std_logic_1164.all;

entity lowpt_tracks is
    port(
		link0_array : IN std_logic_vector(195 downto 0);
		link2_array : IN std_logic_vector(195 downto 0);
		link3_array : IN std_logic_vector(195 downto 0);
		link4_array : IN std_logic_vector(195 downto 0);
		link5_array : IN std_logic_vector(195 downto 0);
		link6_array : IN std_logic_vector(195 downto 0);          
        m: out std_logic_vector(703 downto 0)
    );
end lowpt_tracks;

architecture track_arch of lowpt_tracks is
component singlets
port(
     link0 : in std_logic_vector(195 downto 0);
     link2 : in std_logic_vector(195 downto 0);
     link3 : in std_logic_vector(195 downto 0);
     link4 : in std_logic_vector(195 downto 0);
     link5 : in std_logic_vector(195 downto 0);
     link6 : in std_logic_vector(195 downto 0);

     ai : out std_logic_vector(45 downto 2);
     ao : out std_logic_vector(46 downto 3);
     bi : out std_logic_vector(55 downto 4);
     bo : out std_logic_vector(56 downto 5);
     ci : out std_logic_vector(63 downto 8);
     co : out std_logic_vector(64 downto 9);     
     di : out std_logic_vector(70 downto 13);
     do : out std_logic_vector(71 downto 14);
     ei : out std_logic_vector(75 downto 20);
     eo : out std_logic_vector(76 downto 21);
     fi : out std_logic_vector(80 downto 27);
     fo : out std_logic_vector(81 downto 28);
     gi : out std_logic_vector(82 downto 37);
     go : out std_logic_vector(83 downto 38);     
     hi : out std_logic_vector(88 downto 43);
     ho : out std_logic_vector(89 downto 44)     
);
end component;
COMPONENT doublets
	PORT(
		ai : IN std_logic_vector(45 downto 2);
		ao : IN std_logic_vector(46 downto 3);
		bi : IN std_logic_vector(55 downto 4);
		bo : IN std_logic_vector(56 downto 5);
		ci : IN std_logic_vector(63 downto 8);
		co : IN std_logic_vector(64 downto 9);
		di : IN std_logic_vector(70 downto 13);
		do : IN std_logic_vector(71 downto 14);
		ei : IN std_logic_vector(75 downto 20);
		eo : IN std_logic_vector(76 downto 21);
		fi : IN std_logic_vector(80 downto 27);
		fo : IN std_logic_vector(81 downto 28);
		gi : IN std_logic_vector(82 downto 37);
		go : IN std_logic_vector(83 downto 38);
		hi : IN std_logic_vector(88 downto 43);
		ho : IN std_logic_vector(89 downto 44);          
		a : OUT std_logic_vector(46 downto 3);
		b : OUT std_logic_vector(56 downto 5);
		c : OUT std_logic_vector(64 downto 9);
		d : OUT std_logic_vector(71 downto 14);
		e : OUT std_logic_vector(76 downto 21);
		f : OUT std_logic_vector(81 downto 28);
		g : OUT std_logic_vector(83 downto 38);
		h : OUT std_logic_vector(89 downto 44)
		);
END COMPONENT;
COMPONENT low_tracks
    port(
        a: in std_logic_vector(46 downto 3);
        b: in std_logic_vector(56 downto 5);
        c: in std_logic_vector(64 downto 9);
        d: in std_logic_vector(71 downto 14);
        e: in std_logic_vector(76 downto 21);
        f: in std_logic_vector(81 downto 28);
        g: in std_logic_vector(83 downto 38);
        h: in std_logic_vector(89 downto 44);
        m: out std_logic_vector(703 downto 0)
    );
END COMPONENT;
signal     ai : std_logic_vector(45 downto 2);
signal     ao : std_logic_vector(46 downto 3);
signal     bi : std_logic_vector(55 downto 4);
signal     bo : std_logic_vector(56 downto 5);
signal     ci : std_logic_vector(63 downto 8);
signal     co : std_logic_vector(64 downto 9);     
signal     di : std_logic_vector(70 downto 13);
signal     do : std_logic_vector(71 downto 14);
signal     ei : std_logic_vector(75 downto 20);
signal     eo : std_logic_vector(76 downto 21);
signal     fi : std_logic_vector(80 downto 27);
signal     fo : std_logic_vector(81 downto 28);
signal     gi : std_logic_vector(82 downto 37);
signal     go : std_logic_vector(83 downto 38);     
signal     hi : std_logic_vector(88 downto 43);
signal     ho : std_logic_vector(89 downto 44);     
signal     a: std_logic_vector(46 downto 3);
signal     b: std_logic_vector(56 downto 5);
signal     c: std_logic_vector(64 downto 9);
signal     d: std_logic_vector(71 downto 14);
signal     e: std_logic_vector(76 downto 21);
signal     f: std_logic_vector(81 downto 28);
signal     g: std_logic_vector(83 downto 38);
signal     h: std_logic_vector(89 downto 44);


       -- signal definitions 

 begin 
I_singlets: singlets PORT MAP(
		link0 => link0_array,
		link2 => link2_array,
		link3 => link3_array,
		link4 => link4_array,
		link5 => link5_array,
		link6 => link6_array,
		ai => ai,
		ao => ao,
		bi => bi,
		bo => bo,
		ci => ci,
		co => co,
		di => di,
		do => do,
		ei => ei,
		eo => eo,
		fi => fi,
		fo => fo,
		gi => gi,
		go => go,
		hi => hi,
		ho => ho
	);
I_doublets: doublets PORT MAP(
		ai => ai,
		ao => ao,
		bi => bi,
		bo => bo,
		ci => ci,
		co => co,
		di => di,
		do => do,
		ei => ei,
		eo => eo,
		fi => fi,
		fo => fo,
		gi => gi(82 downto 37),
		go => go,
		hi => hi,
		ho => ho,
		a => a,
		b => b,
		c => c,
		d => d,
		e => e,
		f => f,
		g => g,
		h => h
	);
I_low_tracks: low_tracks PORT MAP(
		a => a,
		b => b,
		c => c,
		d => d,
		e => e,
		f => f,
		g => g,
		h => h,
		m => m
	);
end track_arch;
