library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.dfea_pack.all;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity back_chip is
    Port ( clk_in : in std_logic;
-- link0 and link2 data now comes from U7 using DDR IO of VIRTEXII
        	  link02:     in std_logic_vector(27 downto 0);
-- link3 and link4 data now comes from U7 using DDR IO of VIRTEXII
        	  link34:     in std_logic_vector(27 downto 0);
-- link5 and link6 data now comes from U9 using DDR IO of VIRTEXII
        	  link56:     in std_logic_vector(27 downto 0);

           low_muon_trk:   in std_logic_vector(10 downto 0);	-- format VCEeePppppp
           low_ctoc_trk:   in std_logic_vector(10 downto 0);  -- format VCEeePppppp
           rst_dcm:	  in std_logic;
           lock_out:	  out std_logic;
           eof:	  in std_logic;
           bus_cmd:	  in std_logic;
		     ctrl_bus:   inout std_logic_vector(15 downto 0);					
-- test pins
			  diag:		out	std_logic_vector(3 downto 0);	-- diagnostic header JP3 pins 9 thru 12
			  prev_trk:	in	std_logic;						-- used for L1 isolation
			  next_trk:	in	std_logic;
			  home_trka:	out	std_logic;
			  home_trkb:	out	std_logic;
-- muon and level 1/2 output
           link0_clk : out std_logic;
           link0_out : out std_logic_vector(27 downto 0);
           link1_clk : out std_logic;
           link1_out : out std_logic_vector(27 downto 0);
			  muon_dat: 	out	std_logic_vector(15 downto 0);	-- list of tracks to Muon L1 trig
			  muon_clk:	out	std_logic;						-- clock for Muon SLDB
			  muon_xe:	out	std_logic;						-- XMIT ENABLE control bit for Muon SLDB
			  muon_pe:	out	std_logic						-- PARITY ENABLE control bit for Muon SLDB
			  );
end back_chip;

architecture my_arch of back_chip is
COMPONENT clocks
	PORT(
		clk_in : IN std_logic;          
		reset : IN std_logic;          
		clk : OUT std_logic;
		clkn : OUT std_logic;
		clk270 : OUT std_logic;
		lock : OUT std_logic
		);
END COMPONENT;
COMPONENT ddr_demux
	PORT(
		clk : IN std_logic;
		clkn : IN std_logic;
		load : IN std_logic;
		ddr_link : IN std_logic_vector(27 downto 0);          
		q_a : OUT std_logic_vector(195 downto 0);
		q_b : OUT std_logic_vector(195 downto 0);
		link_a : OUT std_logic_vector(27 downto 0);
		link_b : OUT std_logic_vector(27 downto 0)
		);
END COMPONENT;
component backend
            port (
                matrix_d:   in	std_logic_vector(351 downto 0);      -- inputs for the matrix
                clock:      in 	std_logic;                           -- runs at 53MHz (except when loading)
                reset:      in 	std_logic;                           -- active low async
                load:       in 	std_logic;                           -- the crossing marker, delayed by n clk cycles.
                muon_trk:   out std_logic_vector(9 downto 0);
				ctoc_trk:	out std_logic_vector(9 downto 0)
             );
end component;
component highpt_tracks
            port (
                link0_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link2_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link3_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link4_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link5_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link6_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                eqs_vers:  out std_logic_vector(31 downto 0);      -- high equation version and sector number
                m:  out std_logic_vector(351 downto 0)      -- matrix values out
            );
end component;
component maxpt_tracks
            port (
                link0_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link2_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link3_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link4_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link5_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                link6_array:  in  std_logic_vector(195 downto  0);     -- doublet layers in
                eqs_vers:   out std_logic_vector(31 downto 0);   -- max equation version and sector number
                m:  out std_logic_vector(351 downto 0)      -- matrix values out
            );
end component;
COMPONENT medpt_tracks
	PORT(
		link0_array : IN std_logic_vector(195 downto 0);
		link2_array : IN std_logic_vector(195 downto 0);
		link3_array : IN std_logic_vector(195 downto 0);
		link4_array : IN std_logic_vector(195 downto 0);
		link5_array : IN std_logic_vector(195 downto 0);
		link6_array : IN std_logic_vector(195 downto 0);          
		a : OUT std_logic_vector(46 downto 3);
		b : OUT std_logic_vector(56 downto 5);
		c : OUT std_logic_vector(64 downto 9);
		d : OUT std_logic_vector(71 downto 14);
		e : OUT std_logic_vector(76 downto 21);
		f : OUT std_logic_vector(81 downto 28);
		g : OUT std_logic_vector(83 downto 38);
		h : OUT std_logic_vector(89 downto 44);
		eqs_vers : OUT std_logic_vector(31 downto 0);          
		m : OUT std_logic_vector(351 downto 0)
		);
END COMPONENT;
component doublet_counter
            port (
                clock:      in  std_logic;
                load:       in  std_logic;                      -- active high
                a:          in  std_logic_vector(32 downto 17); -- doublets 
                b:          in  std_logic_vector(40 downto 21);
                c:          in  std_logic_vector(48 downto 25);
                d:          in  std_logic_vector(56 downto 29);
                e:          in  std_logic_vector(64 downto 33);
                f:          in  std_logic_vector(72 downto 37);
                g:          in  std_logic_vector(80 downto 41);
                h:          in  std_logic_vector(88 downto 45);
                sum_out:    out std_logic_vector( 2 downto  0)
            );
end component;
component cps_map is
    port(
         link0_array: in std_logic_vector(195 downto 0);
         link2_array: in std_logic_vector(195 downto 0);
         link3_array: in std_logic_vector(195 downto 0);
         link4_array: in std_logic_vector(195 downto 0);
         link5_array: in std_logic_vector(195 downto 0);
         link6_array: in std_logic_vector(195 downto 0);
         cps:         out std_logic_vector(31 downto 0)
    );
end component;
	COMPONENT ctrl_decode
	PORT(
		clock : IN std_logic;
		eof : IN std_logic;
		bus_cmd : IN std_logic;    
		version : IN std_logic_vector(15 downto 0);
		bus_o : IN std_logic_vector(15 downto 0);
		ctrl_bus : INOUT std_logic_vector(15 downto 0);
		r_data : OUT std_logic_vector(15 downto 0);      
		ctrl_q : OUT std_logic_vector(3 downto 0);
		cap_l1 : OUT std_logic;
		cap_l2 : OUT std_logic;
		lock : OUT std_logic;
		reset : OUT std_logic;
		u10 : IN std_logic;
		wr_mobo : OUT std_logic;
		rd_mobo : OUT std_logic;
		rd_eqs_vers : OUT std_logic;
		rd_status : OUT std_logic;
		rd_input : OUT std_logic;
		rd_muon : OUT std_logic;
		rd_CTOC : OUT std_logic;
		diag_sel : OUT std_logic_vector(3 downto 0);
		mem_depth : OUT std_logic_vector(8 downto 0);
		addr : OUT std_logic_vector(9 downto 0);
		w_data : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;
	COMPONENT aa_top
	PORT(
		clock : IN std_logic;
		clock_n : IN std_logic;
		reset : IN std_logic;
		fx : IN std_logic;
		sync_gap : IN std_logic;
		l1 : IN std_logic;
		cft_reset : IN std_logic;
		u10 : IN std_logic;
		wr_mobo : IN std_logic;
		rd_mobo : IN std_logic;
		rd_input : IN std_logic;
		rd_muon : IN std_logic;
		rd_CTOC : IN std_logic;
		cap_l1 : IN std_logic;
		cap_l2 : IN std_logic;
		link_data : IN std_logic_vector(111 downto 0);
		mem_depth : IN std_logic_vector(8 downto 0);
		addr : IN std_logic_vector(9 downto 0);
		w_data : IN std_logic_vector(15 downto 0);
		start : IN std_logic;
		max_muon_trk : IN std_logic_vector(9 downto 0);
		high_muon_trk : IN std_logic_vector(9 downto 0);
		med_muon_trk : IN std_logic_vector(9 downto 0);
		low_muon_trk : IN std_logic_vector(10 downto 0);
		max_ctoc_trk : IN std_logic_vector(9 downto 0);
		high_ctoc_trk : IN std_logic_vector(9 downto 0);
		med_ctoc_trk : IN std_logic_vector(9 downto 0);
		low_ctoc_trk : IN std_logic_vector(10 downto 0);
		cphi : in array_x8y6;
		cluster : IN std_logic_vector(17 downto 0);
		doublet : IN std_logic_vector(2 downto 0);
		prev_trk : IN std_logic;
		next_trk : IN std_logic;          
		r_data : OUT std_logic_vector(15 downto 0);
		valid_trk : OUT std_logic;
		bad_fx : OUT std_logic;
		sg_err : OUT std_logic;
		locked : OUT std_logic;
		diag : OUT std_logic_vector(3 downto 0);
		home_trk : OUT std_logic;
		ctoc_out : OUT std_logic_vector(27 downto 0);
		muon_dat : OUT std_logic_vector(15 downto 0);
		muon_xe : OUT std_logic;
		muon_pe : OUT std_logic
		);
	END COMPONENT;
COMPONENT cluster_new
	PORT(
		clock : IN std_logic;
		load : IN std_logic;
		cps : IN std_logic_vector(31 downto 0);          
		cphi : OUT array_x8y6
		);
END COMPONENT;

component cluster_top
            port(
                cps:        in  std_logic_vector(31 downto 0);
                clock:      in  std_logic;
                load:       in  std_logic;
                reset:      in  std_logic;
                valid_a:    out std_logic;
                width_a:    out std_logic_vector(3 downto 0);
                center_a:   out std_logic_vector(3 downto 0);
                valid_b:    out std_logic;
                width_b:    out std_logic_vector(3 downto 0);
                center_b:   out std_logic_vector(3 downto 0)
            );
end component;
signal matrix_m:      std_logic_vector(351 downto 0);
signal matrix_h:      std_logic_vector(351 downto 0);
signal matrix_n:      std_logic_vector(351 downto 0);
signal mcell:      std_logic_vector(351 downto 0);
signal hcell:      std_logic_vector(351 downto 0);
signal ncell:      std_logic_vector(351 downto 0);
signal z704:      std_logic_vector(703 downto 0);
signal z352:      std_logic_vector(351 downto 0);
signal a : std_logic_vector(46 downto 3);
signal b : std_logic_vector(56 downto 5);
signal c : std_logic_vector(64 downto 9);
signal d : std_logic_vector(71 downto 14);
signal e : std_logic_vector(76 downto 21);
signal f : std_logic_vector(81 downto 28);
signal g : std_logic_vector(83 downto 38);
signal h : std_logic_vector(89 downto 44);
signal cps: std_logic_vector(31 downto 0);
signal cphi : array_x8y6;
signal version: std_logic_vector(15 downto 0);
signal max_vers: std_logic_vector(31 downto 0);
signal high_vers: std_logic_vector(31 downto 0);
signal med_vers: std_logic_vector(31 downto 0);
signal eqs_vers: std_logic_vector(15 downto 0);
signal dcm_lock: std_logic;
signal ctrl_lock: std_logic;
signal clock: std_logic;
signal clock_n: std_logic;
signal clk270: std_logic;
signal clk270_n: std_logic;
signal link_data: std_logic_vector(111 downto 0);
signal link0_array: std_logic_vector(195 downto 0);
signal link2_array: std_logic_vector(195 downto 0);
signal link3_array: std_logic_vector(195 downto 0);
signal link4_array: std_logic_vector(195 downto 0);
signal link5_array: std_logic_vector(195 downto 0);
signal link6_array: std_logic_vector(195 downto 0);
signal max_muon_trk: std_logic_vector(9 downto 0);
signal high_muon_trk: std_logic_vector(9 downto 0);
signal med_muon_trk: std_logic_vector(9 downto 0);
signal max_ctoc_trk: std_logic_vector(9 downto 0);
signal high_ctoc_trk: std_logic_vector(9 downto 0);
signal med_ctoc_trk: std_logic_vector(9 downto 0);
signal cluster: std_logic_vector(17 downto 0);
signal doublets: std_logic_vector(2 downto 0);
signal home_trk: std_logic;
signal fx_embed: std_logic;
signal l1_embed: std_logic;
signal sg_embed: std_logic;
signal scl_embed: std_logic;
signal embed_err: std_logic_vector(3 downto 0);
signal ctrl_q: std_logic_vector(3 downto 0);
signal fx: std_logic;
signal l1: std_logic;
signal sg: std_logic;
signal scl: std_logic;
signal cap_l1: std_logic;
signal cap_l1_dl: std_logic;
signal cap_l2: std_logic;
signal reset: std_logic;
signal eof_reg: std_logic_vector(6 downto 0);
signal valid_trk: std_logic;
signal bad_fx: std_logic;
signal sg_err: std_logic;
signal got_trk: std_logic;
signal got_bad_fx: std_logic;
signal got_sg_err: std_logic;
signal locked: std_logic;
signal wr_mobo: std_logic;
signal rd_mobo: std_logic;
signal rd_eqs_vers: std_logic;
signal rd_status: std_logic;
signal rd_input: std_logic;
signal rd_muon: std_logic;
signal rd_CTOC: std_logic;
signal w_data: std_logic_vector(15 downto 0);
signal r_data: std_logic_vector(15 downto 0);
signal bus_o: std_logic_vector(15 downto 0);
signal mem_depth: std_logic_vector(8 downto 0);
signal addr: std_logic_vector(9 downto 0);
signal ctoc_out: std_logic_vector(27 downto 0);
signal diag0_d0: std_logic;
signal diag0_d1: std_logic;
signal diag_sel: std_logic_vector(3 downto 0);
signal diagp: std_logic_vector(3 downto 0);
signal diag_aa_top: std_logic_vector(3 downto 0);
signal lvds_in: std_logic;
signal lvds_out: std_logic;
attribute iobdelay: string;
attribute iobdelay of eof: signal is "BOTH";
attribute iobdelay of bus_cmd: signal is "BOTH";
attribute iobdelay of ctrl_bus: signal is "BOTH";
attribute iobdelay of low_muon_trk: signal is "BOTH";
attribute iobdelay of low_ctoc_trk: signal is "BOTH";
attribute iobdelay of prev_trk: signal is "BOTH";
attribute iobdelay of next_trk: signal is "BOTH";
attribute drive : string;
attribute drive of ctrl_bus : signal is "8";
attribute drive of lock_out : signal is "8";
attribute drive of home_trka : signal is "8";
attribute drive of home_trkb : signal is "8";
attribute fast : string;
attribute fast of ctrl_bus : signal is "false";
attribute fast of lock_out : signal is "false";
attribute fast of home_trka : signal is "false";
attribute fast of home_trkb : signal is "false";
begin
version <= x"000b";
z704 <= (others => '0');
z352 <= (others => '0');
home_trka <= home_trk;
home_trkb <= home_trk;
lock_out <= dcm_lock and ctrl_lock;
i_clocks: clocks PORT MAP(
		clk_in => clk_in,
		reset => rst_dcm,
		clk => clock,
		clkn => clock_n,
		clk270 => clk270,
		lock => dcm_lock
	);
process(clock)
begin
  if(clock'event and clock = '1')then
    if(reset = '0')then
	   link0_out <= (others => '0');
	   link1_out <= (others => '0');
	 else
	   link0_out <= ctoc_out;
	   link1_out <= ctoc_out;
	 end if;
	 eof_reg <= eof_reg(5 downto 0) & eof;
	 bus_o <= r_data;
  end if;
end process;
i_ddr_demux02: ddr_demux PORT MAP(
		clk => clock,
		clkn => clock_n,
		load => eof_reg(2),
		ddr_link => link02,
		q_a => link0_array,
		q_b => link2_array,
		link_a => link_data(27 downto 0),
		link_b => link_data(55 downto 28)
	);
i_ddr_demux34: ddr_demux PORT MAP(
		clk => clock,
		clkn => clock_n,
		load => eof_reg(2),
		ddr_link => link34,
		q_a => link3_array,
		q_b => link4_array,
		link_a => link_data(83 downto 56),
		link_b => link_data(111 downto 84)
	);
i_ddr_demux56: ddr_demux PORT MAP(
		clk => clock,
		clkn => clock_n,
		load => eof_reg(2),
		ddr_link => link56,
		q_a => link5_array,
		q_b => link6_array
	);

track_equations_m: maxpt_tracks
            port map(
                link0_array => link0_array,
                link2_array => link2_array,
                link3_array => link4_array,
                link4_array => link3_array,
                link5_array => link5_array,
                link6_array => link6_array,
					 eqs_vers => max_vers,
                m(351 downto 0) => matrix_m(351 downto 0)
            );
track_equations_h: highpt_tracks
            port map(
                link0_array => link0_array,
                link2_array => link2_array,
                link3_array => link4_array,
                link4_array => link3_array,
                link5_array => link5_array,
                link6_array => link6_array,
					 eqs_vers => high_vers,
                m(351 downto 0)         => matrix_h(351 downto 0)
            );
track_equations_n: medpt_tracks
            port map(
                link0_array => link0_array,
                link2_array => link2_array,
                link3_array => link4_array,
                link4_array => link3_array,
                link5_array => link5_array,
                link6_array => link6_array,
					 eqs_vers => med_vers,
                a     => a,
                b     => b,
                c     => c,
                d     => d,
                e     => e,
                f     => f,
                g     => g,
                h     => h,
                m     => matrix_n
            );
    
i_mveto_general: mveto_general PORT MAP(
		mc => matrix_m,
		hc => matrix_h,
		nc => matrix_n,
		lc => z704,
		ml => z352,
		hl => z352,
		nl => z352,
		ll => z704,
		mr => z352,
		hr => z352,
		nr => z352,
		lr => z704,
		mcell => mcell,
		hcell => hcell,
		ncell => ncell
	);
i_maximum: backend
            port map(
                matrix_d  => mcell,
                clock     => clock,
                reset     => reset,
                load      => eof_reg(3),
					 muon_trk => max_muon_trk,
					 ctoc_trk => max_ctoc_trk
            );
i_high: backend
            port map(
                matrix_d  => hcell,
                clock     => clock,
                reset     => reset,
                load      => eof_reg(3),
					 muon_trk => high_muon_trk,
					 ctoc_trk => high_ctoc_trk
            );
i_medium: backend
            port map(
                matrix_d  => ncell,
                clock     => clock,
                reset     => reset,
                load      => eof_reg(3),
					 muon_trk => med_muon_trk,
					 ctoc_trk => med_ctoc_trk
            );
bit_counter: doublet_counter
            port map(
                clock               => clock, 
                load                => eof_reg(4),
                a(32 downto 17)     => a(32 downto 17),
                b(40 downto 21)     => b(40 downto 21),
                c(48 downto 25)     => c(48 downto 25),
                d(56 downto 29)     => d(56 downto 29),
                e(64 downto 33)     => e(64 downto 33),
                f(72 downto 37)     => f(72 downto 37),
                g(80 downto 41)     => g(80 downto 41),
                h(88 downto 45)     => h(88 downto 45),
                sum_out(2 downto 0) => doublets(2 downto 0)
            );
cps_mapper: cps_map
        port map(
            link0_array(195 downto 0) => link0_array(195 downto 0),
            link2_array(195 downto 0) => link2_array(195 downto 0),
            link3_array(195 downto 0) => link4_array(195 downto 0),
            link4_array(195 downto 0) => link3_array(195 downto 0),
            link5_array(195 downto 0) => link5_array(195 downto 0),
            link6_array(195 downto 0) => link6_array(195 downto 0),
            cps(31 downto 0)          => cps(31 downto 0)
        );

I_cluster_new: cluster_new PORT MAP(
		clock => clock,
		load => eof_reg(0),
		cps => cps,
		cphi => cphi
	);

dual_cluster_finder: cluster_top
            port map(
                cps(31 downto 0)        => cps(31 downto 0),
                clock                   => clock,
                load                    => eof_reg(6),
                reset                   => reset,
                valid_a                 => cluster(17),
                width_a(3 downto 0)     => cluster(16 downto 13),
                center_a(3 downto 0)    => cluster(12 downto  9),
                valid_b                 => cluster(8),
                width_b(3 downto 0)     => cluster(7 downto 4),
                center_b(3 downto 0)    => cluster(3 downto 0)
            );
process(clock)
begin
  if(clock'event and clock = '1')then
	 case addr(2 downto 0) is
		when "010" => eqs_vers <= max_vers(15 downto 0);
		when "011" => eqs_vers <= max_vers(31 downto 16);
		when "100" => eqs_vers <= high_vers(15 downto 0);
		when "101" => eqs_vers <= high_vers(31 downto 16);
		when "110" => eqs_vers <= med_vers(15 downto 0);
		when "111" => eqs_vers <= med_vers(31 downto 16);
		when others => NULL;
	 end case;
  end if;
end process;
r_data <= eqs_vers when rd_eqs_vers = '1' else (others => 'Z');
-- SLDB clock and link_out clocks
clk270_n <= not clk270;
i_muon_clk:  fddrrse
	      port map(
			    d0 => vcc,
				 d1 => gnd,
				 c0 => clk270,
				 c1 => clk270_n,
				 ce => vcc,
				 r  => gnd,
				 s  => gnd,
				 q  => muon_clk);
i_link0_clk:  fddrrse
	      port map(
			    d0 => vcc,
				 d1 => gnd,
				 c0 => clock,
				 c1 => clock_n,
				 ce => vcc,
				 r  => gnd,
				 s  => gnd,
				 q  => link0_clk);
i_link1_clk:  fddrrse
	      port map(
			    d0 => vcc,
				 d1 => gnd,
				 c0 => clock,
				 c1 => clock_n,
				 ce => vcc,
				 r  => gnd,
				 s  => gnd,
				 q  => link1_clk);
-- backplane interface
process(clock)
begin
  if(clock'event and clock = '1')then
    if(eof_reg(1) = '1')then
      fx <= ctrl_q(3);
      sg <= ctrl_q(2);
      l1 <= ctrl_q(1);
      scl <= ctrl_q(0);
	 end if;
	 cap_l1_dl <= cap_l1;
  end if;
end process;
i_ctrl_decode: ctrl_decode PORT MAP(
		clock => clock,
		eof => eof_reg(6),
		bus_cmd => bus_cmd,
		bus_o => bus_o,
		version => version,
		ctrl_bus => ctrl_bus,
		cap_l1 => cap_l1,
		cap_l2 => cap_l2,
		ctrl_q => ctrl_q,
		lock => ctrl_lock,
		reset => reset,
		u10 => gnd,
		wr_mobo => wr_mobo,
		rd_mobo => rd_mobo,
		rd_eqs_vers => rd_eqs_vers,
		rd_status => rd_status,
		rd_input => rd_input,
		rd_muon => rd_muon,
		rd_CTOC => rd_CTOC,
		diag_sel => diag_sel,
		mem_depth => mem_depth,
		addr => addr,
		w_data => w_data,
		r_data => r_data
	);
i_aa_top: aa_top PORT MAP(
		clock => clock,
		clock_n => clock_n,
		reset => reset,
		fx => fx,
		sync_gap => sg,
		l1 => l1,
		cft_reset => scl,
		u10 => gnd,
		wr_mobo => wr_mobo,
		rd_mobo => rd_mobo,
		rd_input => rd_input,
		rd_muon => rd_muon,
		rd_CTOC => rd_CTOC,
		cap_l1 => cap_l1_dl,
		cap_l2 => cap_l2,
		link_data => link_data,
		mem_depth => mem_depth,
		addr => addr,
		w_data => w_data,
		r_data => r_data,
		start => eof_reg(2),
		max_muon_trk => max_muon_trk,
		high_muon_trk => high_muon_trk,
		med_muon_trk => med_muon_trk,
		low_muon_trk => low_muon_trk,
		max_ctoc_trk => max_ctoc_trk,
		high_ctoc_trk => high_ctoc_trk,
		med_ctoc_trk => med_ctoc_trk,
		low_ctoc_trk => low_ctoc_trk,
		cphi => cphi,
		cluster => cluster,
		doublet => doublets,
		valid_trk => valid_trk,
		bad_fx => bad_fx,
		sg_err => sg_err,
		locked => locked,
		diag => diag_aa_top,
		prev_trk => prev_trk,
		next_trk => next_trk,
		home_trk => home_trk,
		ctoc_out => ctoc_out,
		muon_dat => muon_dat,
		muon_xe => muon_xe,
		muon_pe => muon_pe
	);
-- compare embedded control bits with backplane control bits
process(clock)
begin
  if(clock'event and clock = '1')then
    if(eof_reg(4) = '1')then
	   scl_embed <= link2_array(57);
	   l1_embed <= link2_array(113);
	   sg_embed <= link2_array(29);
	   fx_embed <= link2_array(1);
	   embed_err(0) <= scl xor scl_embed;
	   embed_err(1) <= l1 xor l1_embed;
	   embed_err(2) <= sg xor sg_embed;
	   embed_err(3) <= fx xor fx_embed;
	 end if;
	 got_trk <= valid_trk or (got_trk and not rd_status);
	 got_bad_fx <= bad_fx or (got_bad_fx and not rd_status);
	 got_sg_err <= sg_err or (got_sg_err and not rd_status);
  end if;
end process;
process(clock)
begin
  if(clock'event and clock = '1')then
	 case diag_sel is
	   when "0001" =>
		  diagp(0) <= l1_embed;
		  diagp(1) <= l1;
		  diagp(2) <= fx_embed;
		  diagp(3) <= fx;
	   when "0010" =>
		  diagp(0) <= sg_embed;
		  diagp(1) <= sg;
		  diagp(2) <= scl_embed;
		  diagp(3) <= scl;
	   when "0011" =>
		  diagp(0) <= diag_aa_top(0);
		  diagp(1) <= diag_aa_top(1);
		  diagp(2) <= diag_aa_top(2);
		  diagp(3) <= diag_aa_top(3);
	   when "0100" =>
		  diagp(0) <= eof_reg(4);
		  diagp(1) <= dcm_lock;
		  diagp(2) <= gnd;
		  diagp(3) <= gnd;
	   when "0101" =>
		  diagp(0) <= eof_reg(4);
		  diagp(1) <= eof_reg(4);
		  diagp(2) <= eof_reg(4);
		  diagp(3) <= eof_reg(4);
	   when others =>
		  diagp(0) <= gnd;
		  diagp(1) <= gnd;
		  diagp(2) <= gnd;
		  diagp(3) <= gnd;
	 end case;
	 diag(3 downto 1) <= diagp(3 downto 1);
  end if;
end process;
-- output clock signal when diag_sel = "1111"
diag0_d0 <= '1' when diag_sel = "1111" else diagp(0);
diag0_d1 <= '0' when diag_sel = "1111" else diagp(0);
diag0:  fddrrse
	      port map(
			    d0 => diag0_d0,
				 d1 => diag0_d1,
				 c0 => clock,
				 c1 => clock_n,
				 ce => vcc,
				 r  => gnd,
				 s  => gnd,
				 q  => diag(0));
r_data <= "000000" & lvds_out & lvds_in & embed_err & got_bad_fx & got_sg_err & got_trk & locked
          when rd_status = '1' else (others => 'Z');
process(clock)
begin
  if(clock'event and clock = '1')then
    if(eof_reg(5) = '1')then
	   lvds_in <= prev_trk;
	   lvds_out <= home_trk;
	 end if;
  end if;
end process;
end my_arch;
