-- cluster_match.vhd
-- 28 Jan 2002
-- jamieson olsen <jamieson@fnal.gov>

--	track(5) 	= set if valid track
--  track(4..0)	= track phi projected up to CPS strips

-- 	cphix(5)	= set if valid cluster centroid
--  cphix(4..0)	= centroid of CPS cluster

--  CPS strips as follows: 0-7 are PREV; 8-15 are HOME; 16-23 are NEXT.
--  numbers increase with PHI.

-- 	cluster centroids are applied to this module in parallel, tracks are
-- 	presented serially.  It does not matter in which order clusters* or tracks
--  are presented to this module.  This logic is PIPELINED; latency is 3 clks.
-- 	this modules accepts tracks from a single Pt threshold bin.  Four of these
-- 	modules are needed in DFEA U1 FPGA.

-- 	see my engineering note 2002-01-25b for block diagram...

-- 	modified 21 Oct 2002: added reset.
--  13 May 2003: curve set if CCW.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity cluster_match is
	port (
		clock:		in 	std_logic;
		reset:		in	std_logic;						-- active low
		track:		in	std_logic_vector(5 downto 0);	-- valid + projected track phi
		start:		in	std_logic;						-- set when 1st track is clocked in
		curve:		in	std_logic;						-- CFT track curvature bit, set if CCW
		window:		in	std_logic_vector(3 downto 0);	-- match window, in CPS strips.

		cphi0:		in	std_logic_vector(5 downto 0);	-- cluster 0
		cphi1:		in	std_logic_vector(5 downto 0);	-- cluster 1
		cphi2:		in	std_logic_vector(5 downto 0);	-- cluster 2 , etc.
		cphi3:		in	std_logic_vector(5 downto 0);
		cphi4:		in	std_logic_vector(5 downto 0);
		cphi5:		in	std_logic_vector(5 downto 0);
		cphi6:		in	std_logic_vector(5 downto 0);
		cphi7:		in	std_logic_vector(5 downto 0);

		cnt_cw:			out	std_logic_vector(2 downto 0);	-- count of CW  curving tracks w/  cluster match
		cnt_ccw:		out	std_logic_vector(2 downto 0);	-- count of CCW curving tracks w/  cluster match
		cnt_ncw:		out	std_logic_vector(2 downto 0);	-- count of CW  curving tracks w/o cluster match
		cnt_nccw:		out	std_logic_vector(2 downto 0);	-- count of CCW curving tracks w/o cluster match
		cnt_valid:		out	std_logic;						-- set when the counts are valid

		cluster_index:	out	std_logic_vector(2 downto 0);
--		track_index:	out	std_logic_vector(2 downto 0);
		valid_match:	out	std_logic
	);
end cluster_match;

architecture cluster_match_arch of cluster_match is

	-- declare registers

--		signal track_index_reg:		std_logic_vector(2 downto 0);
		signal delay_start:			std_logic_vector(3 downto 0);
		signal delay_curve:			std_logic_vector(3 downto 0);
		signal delay_valid:			std_logic_vector(3 downto 0);

	-- declare signals
	
		signal diff0_out: 		std_logic_vector(4 downto 0);
		signal diff1_out: 		std_logic_vector(4 downto 0);
		signal diff2_out: 		std_logic_vector(4 downto 0);
		signal diff3_out: 		std_logic_vector(4 downto 0);
		signal diff4_out: 		std_logic_vector(4 downto 0);
		signal diff5_out: 		std_logic_vector(4 downto 0);
		signal diff6_out: 		std_logic_vector(4 downto 0);
		signal diff7_out: 		std_logic_vector(4 downto 0);

		signal comp_01_out:		std_logic_vector(5 downto 0);
		signal comp_23_out:		std_logic_vector(5 downto 0);
		signal comp_45_out:		std_logic_vector(5 downto 0);
		signal comp_67_out:		std_logic_vector(5 downto 0);

		signal comp_0123_out:	std_logic_vector(6 downto 0);
		signal comp_4567_out:	std_logic_vector(6 downto 0);

		signal track_cluster_match:	std_logic;


	-- declare components...

		component diff
			port(
				clock:		in	std_logic;
				reset: 		in	std_logic;
				cft: 		in 	std_logic_vector(5 downto 0);
				cps:		in	std_logic_vector(5 downto 0);
				abs_out:	out	std_logic_vector(4 downto 0)	-- V+abs(3..0)
			);
		end component;
		
		component comp_a
			port(
				clock:		in	std_logic;
				reset: 		in	std_logic;
				a: 			in 	std_logic_vector(4 downto 0);
				b:			in	std_logic_vector(4 downto 0);
				q:			out	std_logic_vector(5 downto 0)
			);
		end component;

		component comp_b
			port(
				clock:		in	std_logic;
				reset: 		in	std_logic;
				a: 			in 	std_logic_vector(5 downto 0);
				b:			in	std_logic_vector(5 downto 0);
				q:			out	std_logic_vector(6 downto 0)
			);
		end component;

		component window_comp is
			port(
				clock:			in	std_logic;
				reset: 			in	std_logic;
				a: 				in 	std_logic_vector(6 downto 0);
				b:				in	std_logic_vector(6 downto 0);
				window:			in	std_logic_vector(3 downto 0);
				cluster_index:	out	std_logic_vector(2 downto 0);
				valid_match:	out	std_logic
			);
		end component;

		component match_counter
			port(
				clock:			in	std_logic;
				reset: 			in	std_logic;
				track_start:	in 	std_logic;		
				track_curve:	in 	std_logic;
				track_valid:	in	std_logic;
				valid_match:	in	std_logic;
				cnt_cw:			out	std_logic_vector(2 downto 0);
				cnt_ccw:		out	std_logic_vector(2 downto 0);
				cnt_ncw:		out	std_logic_vector(2 downto 0);
				cnt_nccw:		out	std_logic_vector(2 downto 0);
				cnt_valid:		out	std_logic
			);
		end component;


 begin

	-- combinatorial stuff...

		valid_match <= track_cluster_match;

--		track_index(2 downto 0) <= track_index_reg(2 downto 0);

	-- sequential stuff...

--		process(clock, reset)		change to sync reset Aug-06-04
		process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				delay_start(3 downto 0) <= "0000";
				delay_curve(3 downto 0) <= "0000";
				delay_valid(3 downto 0) <= "0000";
			 else
				delay_start(3 downto 1)	<= delay_start(2 downto 0);
				delay_start(0)			<= start;

				delay_curve(3 downto 1)	<= delay_curve(2 downto 0);
				delay_curve(0)			<= curve;

				delay_valid(3 downto 1)	<= delay_valid(2 downto 0);
				delay_valid(0)			<= track(5);

--				if (delay_start(2)='1') then
--					track_index_reg(2 downto 0)	<= "000";
--				else
--					track_index_reg(2 downto 0)	<= track_index_reg(2 downto 0) + 1;			
--				end if;

			 end if;
		  end if;
		end process;

	-- instantiate components...

		diff0: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi0(5 downto 0),
				abs_out(4 downto 0)	=> diff0_out(4 downto 0)
			);

		diff1: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi1(5 downto 0),
				abs_out(4 downto 0)	=> diff1_out(4 downto 0)
			);

		diff2: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi2(5 downto 0),
				abs_out(4 downto 0)	=> diff2_out(4 downto 0)
			);

		diff3: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi3(5 downto 0),
				abs_out(4 downto 0)	=> diff3_out(4 downto 0)
			);

		diff4: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi4(5 downto 0),
				abs_out(4 downto 0)	=> diff4_out(4 downto 0)
			);

		diff5: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi5(5 downto 0),
				abs_out(4 downto 0)	=> diff5_out(4 downto 0)
			);

		diff6: diff
			port map(
				clock				=> clock,
				reset 				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi6(5 downto 0),
				abs_out(4 downto 0)	=> diff6_out(4 downto 0)
			);

		diff7: diff
			port map(
				clock				=> clock,
				reset				=> reset,
				cft(5 downto 0)		=> track(5 downto 0),
				cps(5 downto 0)		=> cphi7(5 downto 0),
				abs_out(4 downto 0)	=> diff7_out(4 downto 0)
			);

		comp_01: comp_a
			port map(
				clock			=> 	clock,
				reset 			=> reset,
				a(4 downto 0)	=>	diff0_out(4 downto 0),
				b(4 downto 0)	=>	diff1_out(4 downto 0),
				q(5 downto 0)	=>	comp_01_out(5 downto 0)
			);

		comp_23: comp_a
			port map(
				clock			=> 	clock,
				reset 			=> 	reset,
				a(4 downto 0)	=>	diff2_out(4 downto 0),
				b(4 downto 0)	=>	diff3_out(4 downto 0),
				q(5 downto 0)	=>	comp_23_out(5 downto 0)
			);

		comp_45: comp_a
			port map(
				clock			=> 	clock,
				reset 			=> 	reset,
				a(4 downto 0)	=>	diff4_out(4 downto 0),
				b(4 downto 0)	=>	diff5_out(4 downto 0),
				q(5 downto 0)	=>	comp_45_out(5 downto 0)
			);

		comp_67: comp_a
			port map(
				clock			=> 	clock,
				reset 			=>	reset,
				a(4 downto 0)	=>	diff6_out(4 downto 0),
				b(4 downto 0)	=>	diff7_out(4 downto 0),
				q(5 downto 0)	=>	comp_67_out(5 downto 0)
			);

		comp_0123: comp_b
			port map(
				clock			=>  clock,
				reset 			=> 	reset,
				a(5 downto 0)	=>	comp_01_out(5 downto 0),
				b(5 downto 0)	=>	comp_23_out(5 downto 0),
				q(6 downto 0)	=>	comp_0123_out(6 downto 0)
			);

		comp_4567: comp_b
			port map(
				clock			=>  clock,
				reset 			=> 	reset,
				a(5 downto 0)	=>	comp_45_out(5 downto 0),
				b(5 downto 0)	=>	comp_67_out(5 downto 0),
				q(6 downto 0)	=>	comp_4567_out(6 downto 0)
			);

		cluster_track_match: window_comp
			port map(
				clock						=> clock,
				reset 						=> reset,
				a(6 downto 0)				=> comp_0123_out(6 downto 0),
				b(6 downto 0)				=> comp_4567_out(6 downto 0),
				window(3 downto 0)			=> window(3 downto 0),
				cluster_index(2 downto 0)	=> cluster_index(2 downto 0),
				valid_match					=> track_cluster_match
			);

		level1_counter: match_counter
			port map(
				clock					=> clock,
				reset 					=> reset,
				track_start				=> delay_start(3),
				track_curve				=> delay_curve(3),     	-- set if CCW
				track_valid				=> delay_valid(3),
				valid_match				=> track_cluster_match,
				cnt_cw(2 downto 0)		=> cnt_cw(2 downto 0),
				cnt_ccw(2 downto 0)		=> cnt_ccw(2 downto 0),
				cnt_ncw(2 downto 0)		=> cnt_ncw(2 downto 0),
				cnt_nccw(2 downto 0)	=> cnt_nccw(2 downto 0),
				cnt_valid				=> cnt_valid
			);


end cluster_match_arch;
