-- match_counter.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 28 Jan 2002

-- generates counts for L1 output of DFEA U1 device.  supports up to 6/7 tracks per crossing.

-- revised 21 Oct 2002: added reset
-- 13 May 2003: curve = 1 -> CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity match_counter is
	port(
		clock:			in	std_logic;
		reset:			in	std_logic;
		track_start:	in 	std_logic;		-- set if first track of a crossing
		track_curve:	in 	std_logic;		-- cft track curvature bit, set if CCW
		track_valid:	in	std_logic;		-- set if track is valid
		valid_match:	in	std_logic;		-- set if the track matched a cluster

		cnt_cw:			out	std_logic_vector(2 downto 0);	-- count of CW  tracks w/ match
		cnt_ccw:		out	std_logic_vector(2 downto 0);	-- count of CCW tracks w/ match
		cnt_ncw:		out	std_logic_vector(2 downto 0);	-- count of CW  tracks w/o match
		cnt_nccw:		out	std_logic_vector(2 downto 0);	-- count of CCW tracks w/o match
		cnt_valid:		out	std_logic
	);
end match_counter;

architecture match_counter_arch of match_counter is

	-- define registers...

		signal cw_reg: 			std_logic_vector(2 downto 0);
		signal ccw_reg: 		std_logic_vector(2 downto 0);
		signal ncw_reg: 		std_logic_vector(2 downto 0);
		signal nccw_reg: 		std_logic_vector(2 downto 0);
		signal valid_reg:		std_logic;
		signal free_counter:	std_logic_vector(2 downto 0);
	
	-- define signals

 begin

	-- combinatorial stuff...

		cnt_cw(2 downto 0)		<= cw_reg(2 downto 0);
		cnt_ccw(2 downto 0)		<= ccw_reg(2 downto 0);
		cnt_ncw(2 downto 0)		<= ncw_reg(2 downto 0);
		cnt_nccw(2 downto 0)	<= nccw_reg(2 downto 0);
		cnt_valid				<= valid_reg;

	-- sequential stuff...

--		process(clock, reset) 	 	change to sync reset Aug-06-04
		process(clock) 
		 begin



			if rising_edge(clock) then
			 if (reset='0') then
				cw_reg(2 downto 0) <= "000";
				ccw_reg(2 downto 0) <= "000";
				ncw_reg(2 downto 0) <= "000";
				nccw_reg(2 downto 0) <= "000";
				valid_reg	<= '0';
				free_counter(2 downto 0) <= "000";
			 else

				if (track_start='1') then
					free_counter(2 downto 0) <= "000";
				else
					free_counter(2 downto 0) <= free_counter(2 downto 0) + 1;	
				end if;

				if (free_counter(2 downto 0)="101") then
					valid_reg <= '1';
				else
					valid_reg <= '0';
				end if;

				if (track_start='1') then
					if (track_valid='1' and track_curve='0' and valid_match='1') then
						cw_reg(2 downto 0) 	<= "001";
					else
						cw_reg(2 downto 0) 	<= "000";
					end if;
				else
					if (track_valid='1' and track_curve='0' and valid_match='1') then
						cw_reg(2 downto 0) 	<= cw_reg(2 downto 0) + 1;
					else
						cw_reg(2 downto 0) 	<= cw_reg(2 downto 0);
					end if;
				end if;

				if (track_start='1') then
					if (track_valid='1' and track_curve='1' and valid_match='1') then
						ccw_reg(2 downto 0) 	<= "001";
					else
						ccw_reg(2 downto 0) 	<= "000";
					end if;
				else
					if (track_valid='1' and track_curve='1' and valid_match='1') then
						ccw_reg(2 downto 0) 	<= ccw_reg(2 downto 0) + 1;
					else
						ccw_reg(2 downto 0) 	<= ccw_reg(2 downto 0);
					end if;
				end if;

				if (track_start='1') then
					if (track_valid='1' and track_curve='0' and valid_match='0') then
						ncw_reg(2 downto 0) 	<= "001";
					else
						ncw_reg(2 downto 0) 	<= "000";
					end if;
				else
					if (track_valid='1' and track_curve='0' and valid_match='0') then
						ncw_reg(2 downto 0) 	<= ncw_reg(2 downto 0) + 1;
					else
						ncw_reg(2 downto 0) 	<= ncw_reg(2 downto 0);
					end if;
				end if;

				if (track_start='1') then
					if (track_valid='1' and track_curve='1' and valid_match='0') then
						nccw_reg(2 downto 0) 	<= "001";
					else
						nccw_reg(2 downto 0) 	<= "000";
					end if;
				else
					if (track_valid='1' and track_curve='1' and valid_match='0') then
						nccw_reg(2 downto 0) 	<= nccw_reg(2 downto 0) + 1;
					else
						nccw_reg(2 downto 0) 	<= nccw_reg(2 downto 0);
					end if;
				end if;

			 end if;
		  end if;
		end process;
														
end match_counter_arch;

