-- SRL16x16e.vhd
-- 16-bit shift register; depth is variable from 1 to 16 clock ticks 
-- (can be resized dynamically, too)
-- constructed from SRL16E Virtex primitives.
-- jamieson olsen 10 October 2002

library ieee;
use ieee.std_logic_1164.all;

--library virtex;

entity SRL16x16e is
 	port (
		clock:		in	std_logic;
		ce:			in	std_logic;						-- active high chip enable
		d:			in	std_logic_vector(15 downto 0);	-- data input
		depth:		in	std_logic_vector(3 downto 0);	-- 1 to 16 clocks deep

		q:			out	std_logic_vector(15 downto 0)	-- input delay, delayed by (depth+1) clock ticks
	);
end SRL16x16e;

architecture SRL16x16e_arch of SRL16x16e is

	-- declare components...

	component SRL16E		-- Virtex primative = 1 CLB
		port(
			d:		in	std_logic;
			ce:		in	std_logic;
			clk:	in	std_logic;
			a0:		in	std_logic;
			a1:		in	std_logic;
			a2:		in	std_logic;
			a3:		in	std_logic;
			q:		out	std_logic
		);
	end component;

attribute box_type : string;
attribute box_type of SRL16E : component is "black_box";

begin

	-- instantiate components...

	bit0: SRL16E
		port map(
			d   => d(0),
			q   => q(0),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit1: SRL16E
		port map(
			d   => d(1),
			q   => q(1),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit2: SRL16E
		port map(
			d   => d(2),
			q   => q(2),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit3: SRL16E
		port map(
			d   => d(3),
			q   => q(3),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit4: SRL16E
		port map(
			d   => d(4),
			q   => q(4),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit5: SRL16E
		port map(
			d   => d(5),
			q   => q(5),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit6: SRL16E
		port map(
			d   => d(6),
			q   => q(6),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit7: SRL16E
		port map(
			d   => d(7),
			q   => q(7),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit8: SRL16E
		port map(
			d   => d(8),
			q   => q(8),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit9: SRL16E
		port map(
			d   => d(9),
			q   => q(9),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);		

	bit10: SRL16E
		port map(
			d   => d(10),
			q   => q(10),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

	bit11: SRL16E
		port map(
			d   => d(11),
			q   => q(11),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

	bit12: SRL16E
		port map(
			d   => d(12),
			q   => q(12),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

	bit13: SRL16E
		port map(
			d   => d(13),
			q   => q(13),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

	bit14: SRL16E
		port map(
			d   => d(14),
			q   => q(14),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

	bit15: SRL16E
		port map(
			d   => d(15),
			q   => q(15),
			ce  => ce,
			clk => clock,
			a0  => depth(0),
			a1  => depth(1),
			a2  => depth(2),
			a3  => depth(3)		
		);

end SRL16x16e_arch;
