-- pipe_add.vhd
-- pipelined adder.  accepts six 6-bit numbers.
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***
-- 11 jan 2002: created, simulated OK.
-- 15 may 2002: add a clock cycle delay to be consistant with eng. note 2002-02-08a
-- 22 may 2002: remove the extra clock delay.
-- 07 aug 2002:	mux out the 8 bit sum onto a three bit bus as follows:
--				sum(7..6); sum(5..4); sum(3..2); sum(1..0)
--				msb of the bus is a valid marker.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity pipe_add is
 	port (
		clock:		in 	std_logic;
		load:		in	std_logic;
		d: 			in 	std_logic_vector(35 downto 0);
		q: 			out std_logic_vector( 2 downto 0)
	);
end pipe_add;

architecture pipe_arch of pipe_add is

	-- define registers...

		signal sum_reg:		std_logic_vector( 7 downto 0);
		signal hold_reg:	std_logic_vector( 7 downto 0);
		signal out_reg:		std_logic_vector( 2 downto 0);
		signal in_reg:		std_logic_vector(35 downto 0);
		signal load_reg:	std_logic_vector( 4 downto 0);

	-- define signals...

		signal temp_sum:	std_logic_vector(7 downto 0);

begin

	-- combinatorial stuff...

		q(2 downto 0) 			<= out_reg(2 downto 0);

		temp_sum(7 downto 0)	<= sum_reg(7 downto 0) + ("00" & in_reg(35 downto 30));

	-- sequential stuff...

	adder_pipeline: process(clock)
     begin

		if rising_edge(clock) then

			load_reg(4 downto 1) <= load_reg(3 downto 0);
			load_reg(0) 		 <= load;

			if (load='1') then
				in_reg(35 downto 0) <= d(35 downto 0);
			else
				in_reg(35 downto 30) <= in_reg(29 downto 24);
				in_reg(29 downto 24) <= in_reg(23 downto 18);
				in_reg(23 downto 18) <= in_reg(17 downto 12);
				in_reg(17 downto 12) <= in_reg(11 downto  6);
				in_reg(11 downto  6) <= in_reg( 5 downto  0);
				in_reg( 5 downto  0) <= "000000";
			end if;

			if (load_reg(0)='1') then
				sum_reg(7 downto 0)  <= ("00" & in_reg(35 downto 30));
				hold_reg(7 downto 0) <= sum_reg(7 downto 0);
			else
				sum_reg(7 downto 0)  <= temp_sum(7 downto 0);
			end if;

			if (load_reg(1)='1') then
				out_reg(2 downto 0) <= ('1' & hold_reg(7 downto 6));
			elsif (load_reg(2)='1') then
				out_reg(2 downto 0) <= ('1' & hold_reg(5 downto 4));
			elsif (load_reg(3)='1') then
				out_reg(2 downto 0) <= ('1' & hold_reg(3 downto 2));
			elsif (load_reg(4)='1') then
				out_reg(2 downto 0) <= ('1' & hold_reg(1 downto 0));
			else
				out_reg(2 downto 0) <= "000";
			end if;

		end if;
	end process;

end pipe_arch;
