--   _     _____        _______ ____ _____
--  | |   / _ \ \      / / ____/ ___|_   _|
--  | |  | | | \ \ /\ / /|  _| \___ \ | |  
--  | |__| |_| |\ V  V / | |___ ___) || |  
--  |_____\___/  \_/\_/  |_____|____/ |_|
--                                              
--  CFT AXIAL Track Finder LOW PT bin
--  top level design file
--  jamieson olsen <jamieson@fnal.gov>

-- target device: XCV600-4BG432C

-- *** recent revision history ***

-- 07 jan 2002:	complete overhaul of the front and back end modules to be
--              consistant with the changes made in maximum version 1.5
--
-- 27 feb 2002:	modify for Synplify.
--
-- 21 may 2002:	add status outputs: stat(1) = copy of start bit
-- 				stat(2) = copy of EOF_REG(1)
--
-- 07 aug 2002:	add early l1muon output bus and ctoc_trk bus.  pattern checking for 
--				link6.  see engineering note 2002-08-05a for details.

-- 7 may 2003:	fix curve bit.  1=CCW.


-- EOF timing

-- On each link the "sync bit" is the LSb and it is set only on the LAST word of the
-- seven word frame.  The signal EOF is a delayed copy of the "sync" bit on link2 
-- or link7 of the DFE motherboard.  It is delayed ~ 1 clock cycle with respect to
-- the other links feeding into the board.  
--
-- 01 jul 2004: new run2b design I/O changes:
-- link5 and link6 inputs removed. Receive synchronized link5 and link6 data
-- from U9 via DDR IO
-- synchronized link0 and link2 data sent to U8
-- synchronized link3 and link4 data sent to U8, U9 and U10
-- reset and eof inputs are replaced by ctrl_bus from backplane bus interface
-- link_perr signals are read out via ctrl_bus    S.X.Wu
library ieee,mylib;
use ieee.std_logic_1164.all;
use mylib.dfea_pack.all;
-- library virtex;
-- use virtex.components.all;

entity lowest is
    port(
        link0:      in std_logic_vector(27 downto 0);
        link0_clk:  in std_logic;
        link2:      in std_logic_vector(27 downto 0);
        link2_clk:  in std_logic;
--        link3:      in std_logic_vector(27 downto 0);
--        link3_clk:  in std_logic;
--        link4:      in std_logic_vector(27 downto 0);
--        link4_clk:  in std_logic;
        link5:      in std_logic_vector(27 downto 0);
        link5_clk:  in std_logic;
        link6:      in std_logic_vector(27 downto 0);
        link6_clk:  in std_logic;
-- link3 and link4 data now comes from U7 using DDR IO of VIRTEXII
        link34:     in std_logic_vector(27 downto 0);
-- send link0 and link2 data to U10 using DDR IO of VIRTEXII 
        link02:    out std_logic_vector(27 downto 0);
-- send link5 and link6 data to U10 using DDR IO of VIRTEXII 
        link56:    out std_logic_vector(27 downto 0);
-- send link5 and link6 data to U7 using DDR IO of VIRTEXII 
        link56a:    out std_logic_vector(27 downto 0);
-- send link5 and link6 data to U8 using DDR IO of VIRTEXII 
        link56b:    out std_logic_vector(27 downto 0);
        clk_in:      in std_logic;                       -- DFE board clock, constant 53mhz
-- Although called as eof, it should come one clock earlier than the eof
-- clear signal marks the real end of frame Aug-12-04 
        eof:        in std_logic;  -- sync to clock, active high End of Frame marker.
-- individual start signals are not needed Aug_08-04
--	     start:		  out	std_logic;						-- set to mark 1st muon_trk
        muon_trk:   out std_logic_vector(10 downto 0);	-- format VCEeePppppp
        ctoc_trk:   out std_logic_vector(10 downto 0);  -- format VCEeePppppp
-- in run2b linkx_perr is read out through ctrl_bus
--	     link6_perr: out	std_logic
        diag:    	  out std_logic_vector(3 downto 0);
        rst_dcm:	  in std_logic;
        lock_out:	  out std_logic;
        bus_cmd:	  in std_logic;
		  ctrl_bus:   inout std_logic_vector(15 downto 0)					
    );
end lowest;

architecture lowest_arch of lowest is

    -- define registers...

        signal eof_reg:     std_logic_vector(3 downto 0);

    -- define signals...

--        signal doublet_a:   std_logic_vector(46 downto  3);
--        signal doublet_b:   std_logic_vector(56 downto  5);
--        signal doublet_c:   std_logic_vector(64 downto  9);
--        signal doublet_d:   std_logic_vector(71 downto 14);
--        signal doublet_e:   std_logic_vector(76 downto 21);
--        signal doublet_f:   std_logic_vector(81 downto 28);
--        signal doublet_g:   std_logic_vector(83 downto 38);
--        signal doublet_h:   std_logic_vector(89 downto 44);

    -- define components....
COMPONENT clocks
	PORT(
		clk_in : IN std_logic;          
		reset : IN std_logic;          
		clk : OUT std_logic;
		clkn : OUT std_logic;
		clk270 : OUT std_logic;
		lock : OUT std_logic
		);
END COMPONENT;

COMPONENT read_in
	PORT(
		version : IN std_logic_vector(15 downto 0);
		link1 : IN std_logic_vector(27 downto 0);
		link1_clk : IN std_logic;
		link2 : IN std_logic_vector(27 downto 0);
		link2_clk : IN std_logic;
		link3 : IN std_logic_vector(27 downto 0);
		link3_clk : IN std_logic;
		link4 : IN std_logic_vector(27 downto 0);
		link4_clk : IN std_logic;
		clock : IN std_logic;
		clear : IN std_logic;
		u9 : IN std_logic;
		bus_cmd : IN std_logic;    
		lock : OUT std_logic;
		ctrl_bus : INOUT std_logic_vector(15 downto 0);      
		veto_vers : IN std_logic_vector(15 downto 0);
		eqs_vers : IN std_logic_vector(31 downto 0);
		diag : OUT std_logic_vector(3 downto 0);      
		reset : OUT std_logic;
		link1_out : OUT std_logic_vector(27 downto 0);
		link2_out : OUT std_logic_vector(27 downto 0);
		link3_out : OUT std_logic_vector(27 downto 0);
		link4_out : OUT std_logic_vector(27 downto 0);
		link1_array : OUT std_logic_vector(195 downto 0);
		link2_array : OUT std_logic_vector(195 downto 0);
		link3_array : OUT std_logic_vector(195 downto 0);
		link4_array : OUT std_logic_vector(195 downto 0)
		);
END COMPONENT;
COMPONENT ddr_demux
	PORT(
		clk : IN std_logic;
		clkn : IN std_logic;
		load : IN std_logic;
		ddr_link : IN std_logic_vector(27 downto 0);          
		q_a : OUT std_logic_vector(195 downto 0);
		q_b : OUT std_logic_vector(195 downto 0);
		link_a : OUT std_logic_vector(27 downto 0);
		link_b : OUT std_logic_vector(27 downto 0)
		);
END COMPONENT;
COMPONENT ddr_mux
	PORT(
		clk : IN std_logic;
		clkn : IN std_logic;
		link_a : IN std_logic_vector(27 downto 0);
		link_b : IN std_logic_vector(27 downto 0);          
		link_out : OUT std_logic_vector(27 downto 0)
		);
END COMPONENT;
COMPONENT lowpt_tracks
	PORT(
		link0_array : IN std_logic_vector(195 downto 0);
		link2_array : IN std_logic_vector(195 downto 0);
		link3_array : IN std_logic_vector(195 downto 0);
		link4_array : IN std_logic_vector(195 downto 0);
		link5_array : IN std_logic_vector(195 downto 0);
		link6_array : IN std_logic_vector(195 downto 0);          
--		eqs_vers : OUT std_logic_vector(31 downto 0);          
		m : OUT std_logic_vector(703 downto 0)
		);
END COMPONENT;
  component low_mveto
    port (
      min  : in  std_logic_vector (703 downto 0);
      mout : out std_logic_vector (703 downto 0)
      );
  end component;
component backend32
            port (
                matrix_d:   in std_logic_vector(703 downto 0);      -- inputs for the matrix
                clock:      in std_logic;                           -- runs at 53MHz (except when loading)
                reset:      in std_logic;                           -- active low async
                load:       in std_logic;                           -- the crossing marker, delayed by n clk cycles.
                start:      out std_logic;
                muon_trk:   out std_logic_vector(10 downto 0);
				ctoc_trk:	out std_logic_vector(10 downto 0)
			);
end component;
signal lcell:      std_logic_vector(703 downto 0);
signal matrix_l:      std_logic_vector(703 downto 0);
signal matrix_n:      std_logic_vector(351 downto 0);
signal z704:      std_logic_vector(703 downto 0);
signal z352:      std_logic_vector(351 downto 0);
signal version: std_logic_vector(15 downto 0);
signal eqs_vers: std_logic_vector(31 downto 0);
signal veto_vers: std_logic_vector(15 downto 0);
signal dcm_lock: std_logic;
signal ctrl_lock: std_logic;
signal clock: std_logic;
signal clock_n: std_logic;
signal reset: std_logic;
signal link0_out: std_logic_vector(27 downto 0);
signal link2_out: std_logic_vector(27 downto 0);
signal link2_out_n: std_logic_vector(27 downto 0);
signal link5_out: std_logic_vector(27 downto 0);
signal link6_out: std_logic_vector(27 downto 0);
signal link6_out_n: std_logic_vector(27 downto 0);
signal link0_array: std_logic_vector(195 downto 0);
signal link2_array: std_logic_vector(195 downto 0);
signal link3_array: std_logic_vector(195 downto 0);
signal link4_array: std_logic_vector(195 downto 0);
signal link5_array: std_logic_vector(195 downto 0);
signal link6_array: std_logic_vector(195 downto 0);
attribute clock_buffer: string;
attribute clock_buffer of link0_clk : signal is "bufgp";
attribute clock_buffer of link2_clk : signal is "bufgp";
attribute clock_buffer of link5_clk : signal is "bufgp";
attribute clock_buffer of link6_clk : signal is "bufgp";
attribute iobdelay: string;
attribute iobdelay of eof: signal is "BOTH";
attribute iobdelay of bus_cmd: signal is "BOTH";
attribute iobdelay of ctrl_bus: signal is "BOTH";
attribute drive : string;
attribute drive of ctrl_bus : signal is "8";
attribute drive of lock_out : signal is "8";
attribute fast : string;
attribute fast of ctrl_bus : signal is "false";
attribute fast of lock_out : signal is "false";
begin
version <= x"000b";
z704 <= (others => '0');
z352 <= (others => '0');
i_clocks: clocks PORT MAP(
		clk_in => clk_in,
		reset => rst_dcm,
		clk => clock,
		clkn => clock_n,
		lock => dcm_lock
	);
    -- combinatorial stuff...
lock_out <= dcm_lock and ctrl_lock;

    -- sequential stuff...

        eof_reg_proc: process(clock)
            begin
                if rising_edge(clock) then
--                    eof_reg(1)  <= eof_reg(0);
--                    eof_reg(0)  <= eof;
-- more delays are needed for new DFEA	 S.X.Wu aug-6-2004
						eof_reg <= eof_reg(2 downto 0) & eof;
                end if;
        end process;
    -- instantiate components...
   
        front_end: read_in
            port map(
		version => version,
		link1 => link5,
		link1_clk => link5_clk,
		link2 => link6,
		link2_clk => link6_clk,
		link3 => link0,
		link3_clk => link0_clk,
		link4 => link2,
		link4_clk => link2_clk,
		clock => clock,
		reset => reset,
		veto_vers => veto_vers,
		eqs_vers => eqs_vers,
		clear => eof_reg(0),
		u9 => vcc,
		link1_out => link5_out,
		link2_out => link6_out,
		link3_out => link0_out,
		link4_out => link2_out,
		link1_array => link5_array,
		link2_array => link6_array,
		link3_array => link0_array,
		link4_array => link2_array,
		lock => ctrl_lock,
		diag => diag,
		bus_cmd => bus_cmd,
		ctrl_bus => ctrl_bus
	);
process(clock_n)
begin
  if(clock_n'event and clock_n = '1')then
    link2_out_n <= link2_out;
    link6_out_n <= link6_out;
  end if;
end process;
i_ddr_demux: ddr_demux PORT MAP(
		clk => clock,
		clkn => clock_n,
		load => eof_reg(2),
		ddr_link => link34,
		q_a => link3_array,
		q_b => link4_array
	);
i_ddr_mux02: ddr_mux PORT MAP(
		clk => clock,
		clkn => clock_n,
		link_a => link0_out,
		link_b => link2_out_n,
		link_out => link02
	);
i_ddr_mux56: ddr_mux PORT MAP(
		clk => clock,
		clkn => clock_n,
		link_a => link5_out,
		link_b => link6_out_n,
		link_out => link56
	);
i_ddr_mux56a: ddr_mux PORT MAP(
		clk => clock,
		clkn => clock_n,
		link_a => link5_out,
		link_b => link6_out_n,
		link_out => link56a
	);
i_ddr_mux56b: ddr_mux PORT MAP(
		clk => clock,
		clkn => clock_n,
		link_a => link5_out,
		link_b => link6_out_n,
		link_out => link56b
	);
    
        low_equations: lowpt_tracks
            port map(
		link0_array => link0_array,
		link2_array => link2_array,
		link3_array => link4_array,
		link4_array => link3_array,
		link5_array => link5_array,
		link6_array => link6_array,
--		eqs_vers => eqs_vers,
		m => matrix_l
	);
eqs_vers <= x"00000000";
veto_layer : low_mveto                -- do the veto after the bottom two
                                        -- rows have been eliminated.
    port map(
      min(703 downto 0)  => matrix_l,
      mout(703 downto 0) => lcell
      );
        back_end: backend32
            port map(
                matrix_d(703 downto 0)  => lcell,
                clock                   => clock,
                reset                   => reset,
                load                    => eof_reg(3),
-- individual start signals are not needed Aug_08-04
--                start                   => start,
                muon_trk(10 downto 0)   => muon_trk,
		ctoc_trk(10 downto 0)	=> ctoc_trk
            );

end lowest_arch;
